@@ -10,7 +10,7 @@ our @ISA = qw/Exporter/;
 our @EXPORT_OK = qw( scripts_in_code);
 our @EXPORT = @EXPORT_OK;
 
-our $VERSION = '2.0.98d';
+our $VERSION="2.0.99d";
 
 our @MANIFEST = (
 'clear.sh',
@@ -58,7 +58,7 @@ my %parse_options_low_level_make_sandbox = (
                                 parse => 'd|sandbox_directory=s',      
                                 so    =>  20,
                                 help  => [
-                                            'Where to install the sandbox, under home-directory'
+                                            'Where to install the sandbox, under upper-directory'
                                          ] 
                              },
     sandbox_port          => {
@@ -300,6 +300,16 @@ my %parse_options_replication = (
                                             '(Default: 11000  + version )',
                                          ]
                             },
+    check_base_port      => {
+                                value => undef,               
+                                parse => 'check_base_port',           
+                                so    =>  45,
+                                help  => [
+                                            'Check that the ports are available ', 
+                                            '(Default: disabled )',
+                                         ]
+                            },
+
 
     how_many_slaves      => {
                                 value => 2,               
@@ -340,6 +350,15 @@ my %parse_options_replication = (
                                          ] 
                             },
  
+    interactive           => {
+                                value => 0,                  
+                                parse => 'interactive',                  
+                                so    => 65,
+                                help  => [
+                                            'Use this option to ask interactive user ',
+                                            'confirmation for each node (default: disabled)'
+                                         ] 
+                            },
     verbose               => {
                                 value => 0,                  
                                 parse => 'v|verbose',                  
@@ -394,6 +413,15 @@ my %parse_options_many = (
                                             '(Default: 7000  + version )',
                                          ]
                             },
+    check_base_port      => {
+                                value => undef,               
+                                parse => 'check_base_port',           
+                                so    =>  45,
+                                help  => [
+                                            'Check that the ports are available ', 
+                                            '(Default: disabled )',
+                                         ]
+                            },
 
     how_many_nodes      => {
                                 value => 3,               
@@ -421,6 +449,16 @@ my %parse_options_many = (
                                             'set the nodes in circular replication'
                                          ] 
                             },
+
+    interactive           => {
+                                value => 0,                  
+                                parse => 'interactive',                  
+                                so    => 65,
+                                help  => [
+                                            'Use this option to ask interactive user ',
+                                            'confirmation for each node (default: disabled)'
+                                         ] 
+                            },
     verbose               => {
                                 value => 0,                  
                                 parse => 'v|verbose',                  
@@ -475,7 +513,25 @@ my %parse_options_custom_many = (
                                             '(Default: 5000  + version )',
                                          ]
                             },
+    check_base_port      => {
+                                value => undef,               
+                                parse => 'check_base_port',           
+                                so    =>  45,
+                                help  => [
+                                            'Check that the ports are available ', 
+                                            '(Default: disabled )',
+                                         ]
+                            },
 
+    interactive           => {
+                                value => 0,                  
+                                parse => 'interactive',                  
+                                so    => 65,
+                                help  => [
+                                            'Use this option to ask interactive user ',
+                                            'confirmation for each node (default: disabled)'
+                                         ] 
+                            },
     verbose               => {
                                 value => 0,                  
                                 parse => 'v|verbose',                  
@@ -502,6 +558,9 @@ our %sbtool_supported_operations = (
     copy  => 'copies data from one Sandbox to another',
     move  => 'moves a Sandbox to a different location',
     port  => 'Changes a Sandbox port',
+    delete => 'removes a sandbox completely',
+    preserve => 'makes a sandbox permanent',
+    unpreserve => 'makes a sandbox NOT permanent',
 );
 
 our %sbtool_supported_formats = (
@@ -522,7 +581,7 @@ my %parse_options_sbtool = (
         so    => 20,
         parse => 's|source_dir=s',
         value => undef,
-        help  => 'source directory for move,copy',
+        help  => 'source directory for move, copy, delete',
     },
     dest_dir => {
         so    => 30,
@@ -551,7 +610,7 @@ my %parse_options_sbtool = (
     max_range => {
         so    => 70,
         parse => 'x|max_range=i',
-        value => 32000,
+        value => 64000,
         help  => 'maximum port when searching for available ranges',
     },
     range_size => {
@@ -579,6 +638,12 @@ my %parse_options_sbtool = (
         value => 0,
         help  => 'print more info for "ports" operation'
     },
+    master_node => {
+        so    => 115,
+        parse => 'master_node=i',
+        value => '1',
+        help  => 'which node should be master (default: 1)',
+    },
     tree_nodes => {
         so    => 120,
         parse => 'tree_nodes=s',
@@ -636,7 +701,7 @@ then
 else
     CURDIR=`pwd`
     cd $BASEDIR
-    if [ "$DEBUG" = "" ]
+    if [ "$SBDEBUG" = "" ]
     then
         $MYSQLD_SAFE --defaults-file=$SBDIR/my.sandbox.cnf $@ > /dev/null 2>&1 &
     else
@@ -910,7 +975,7 @@ PERL_SCRIPT3='s/\b$old\b/$new/'
 PERL_SCRIPT="$PERL_SCRIPT1 $PERL_SCRIPT2 $PERL_SCRIPT3"
 
 SCRIPTS1="start stop send_kill clear restart my.sandbox.cnf "
-SCRIPTS2="load_grants my use current_options.conf $0"
+SCRIPTS2="load_grants my use $0"
 SCRIPTS="$SCRIPTS1 $SCRIPTS2"
 for SCRIPT in $SCRIPTS
 do
@@ -951,11 +1016,11 @@ then
 fi
 
 PERL_SCRIPT1='BEGIN{$old=shift;$new=shift};'
-PERL_SCRIPT2='s/\b$old\b/$new/g' 
+PERL_SCRIPT2='s/$old/$new/g' 
 PERL_SCRIPT="$PERL_SCRIPT1 $PERL_SCRIPT2"
 
 SCRIPTS1="start stop send_kill clear restart my.sandbox.cnf "
-SCRIPTS2="load_grants my use current_options.conf $0"
+SCRIPTS2="load_grants my use $0"
 SCRIPTS="$SCRIPTS1 $SCRIPTS2"
 
 for SCRIPT in $SCRIPTS
@@ -973,14 +1038,15 @@ use MySQL::Sandbox;
 
 my $DEBUG = $MySQL::Sandbox::DEBUG;
 
+my $action_list = 'use|start|stop|clear|restart|send_kill';
 my $action = shift
-    or die "action required {use|start|stop|clear}\n";
-$action =~/^(use|start|stop|clear|send_kill)$/ 
-    or die "action must be one of {use|start|stop|clear|send_kill}\n";
+    or die "action required {$action_list}\n";
+$action =~/^($action_list)$/ 
+    or die "action must be one of {$action_list}\n";
 my $sandboxdir = $0;
 $sandboxdir =~ s{[^/]+$}{};
 $sandboxdir =~ s{/$}{};
-my $command = shift;
+my $command = $ARGV[0];
 if ($action eq 'use' and !$command) {
     die "action 'use' requires a command\n";
 }
@@ -1001,11 +1067,11 @@ for my $dir (@dirs) {
         } 
         elsif ( -x "$dir/${action}_all") {
             print "-- executing  $dir/${action}_all\n" if $DEBUG;
-            system("$dir/${action}_all")
+            system("$dir/${action}_all", @ARGV)
         }
         elsif ( -x "$dir/$action") {
             print "-- executing  $dir/$action\n" if $DEBUG;
-            system("$dir/$action")
+            system("$dir/$action", @ARGV)
         }
     }
 }
@@ -1016,7 +1082,7 @@ SANDBOX_ACTION_SCRIPT
 
 my $license_text = <<'LICENSE';
 #    The MySQL Sandbox
-#    Copyright (C) 2006,2007,2008 Giuseppe Maxia, MySQL AB
+#    Copyright (C) 2006,2007,2008,2009 Giuseppe Maxia
 #    Contacts: http://datacharmer.org
 #
 #    This program is free software; you can redistribute it and/or modify
@@ -1063,3 +1129,36 @@ sub scripts_in_code {
 
 1;
 
+=head1 NAME
+
+MySQL::Sandbox::Scripts - Collection of script templates for MySQL Sandbox
+
+=head1 PURPOSE
+
+This package is a collection of data and scripts templates for MySQL::Sandbox.
+It is not supposed to be used stand alone, but it is an ancillary package.
+
+=head1 COPYRIGHT
+
+Version 3.0
+
+Copyright (C) 2006,2007,2008,2009  Giuseppe Maxia
+
+Home Page  http://launchpad.net/mysql-sandbox/
+
+=head1 LEGAL NOTICE
+
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; version 2 of the License.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program; if not, write to the Free Software
+Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301
+USA
+
